using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;

namespace WebsiteCapture
{
    public class ErrorProviderEx : ErrorProvider
    {
        public ErrorProviderEx(IContainer container)
            : base(container)
        {
            container.Add(this);
            this.BlinkStyle = ErrorBlinkStyle.NeverBlink;
        }

        private IContainer components;

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                    components.Dispose();
            }
            base.Dispose(disposing);
        }

        [System.Diagnostics.DebuggerStepThrough()]
        private void InitializeComponent()
        {
            components = new System.ComponentModel.Container();
        }

        private ArrayList _errorList = new ArrayList();
        private Control _activeControl = new Control();

        public new void SetError(Control control, string value)
        {
            if ((value == ""))
            {
                if (_errorList.Contains(control))
                {
                    _errorList.Remove(control);
                }
            }
            else if (!_errorList.Contains(_errorList))
            {
                _errorList.Add(control);
            }
            base.SetError(control, value);
        }

        public void SetError(string value)
        {
            if ((_activeControl == null))
            {
                throw new Exception("Active control has not been set.");
            }
            this.SetError(_activeControl, value);
        }

        public void ClearError(Control control)
        {
            if (_errorList.Contains(control))
            {
                _errorList.Remove(control);
            }
            base.SetError(control, "");
        }

        public void ClearError()
        {
            if ((_activeControl == null))
            {
                throw new Exception("Active control has not been set.");
            }
            this.ClearError(_activeControl);
        }

        public void ClearAll()
        {
            foreach (Control control in _errorList)
            {
                base.SetError(control, "");
            }
            _errorList.Clear();
        }

        public bool HasErrors
        {
            get
            {
                return (_errorList.Count > 0);
            }
        }
    }
}
