using System;
using System.Collections.Generic;
using System.Text;
using Common;
using System.IO;
using System.Windows.Forms;

namespace WebsiteCapture
{
    public class LangManager
    {
        private static LangManager instance;
        
        public static LangManager Instance
        {
            get 
            {
                if (instance == null)
                    instance = new LangManager();
                return instance;
            }
        }

        public enum ELanguage
        {
            English,
            Hungarian,
            German,
        }

        private ELanguage language = ELanguage.English;
        public ELanguage Language
        {
            set 
            {
                this.language = value;
                switch (language)
                {
                    case ELanguage.Hungarian:
                        ini = new IniFile(Path.Combine(System.Windows.Forms.Application.StartupPath, @"Language\Hungarian.ini"));
                        break;
                    case ELanguage.German:
                        ini = new IniFile(Path.Combine(System.Windows.Forms.Application.StartupPath, @"Language\German.ini"));
                        break;
                    case ELanguage.English:
                    default:
                        ini = new IniFile(Path.Combine(System.Windows.Forms.Application.StartupPath, @"Language\Default.ini"));
                        break;
                }
            }
            get { return language; }
        }

        private LangManager()
        {
            this.Language = ELanguage.English;
        }

        private IniFile ini;
        public const string Application = "Application";
        public const string MainForm    = "MainForm";
        public const string Common      = "Common";
        public const string ProcessForm = "ProcessForm";
        public const string URLForm     = "URLForm";
        public const string FragmentForm = "FragmentForm";
        public const string Menu        = "Menu";

        public string Read(string section, string key)
        {
            return ini.IniReadValue(section, key);
        }

        public void Write(string section, string key, string value)
        {
            ini.IniWriteValue(section, key, value);
        }
    }
}
