using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Common;
using System.Collections;
using System.Drawing.Imaging;
using Microsoft.Win32;
using FreeImageAPI;
using System.Runtime.InteropServices;

namespace WebsiteCapture
{
    // skin combo, listview
    // registration
    // help

    public partial class MainForm : Form
    {
        private IconManager iconManager = new IconManager();

        public MainForm()
        {
            InitializeComponent();

            btnCapture.Enabled = btnSelectAll.Enabled = btnDelFile.Enabled = false;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            cmbSize.Items.Clear();
            for (int i = 10; i <= 100; i += 10)
                cmbSize.Items.Add(i);
            if (cmbSize.Items.Count > 0)
                cmbSize.SelectedIndex = cmbSize.Items.Count - 1;

            cmbFormat.Items.Clear();
            cmbFormat.Items.AddRange(OutputFormat.Formats.ToArray());

            txtOutputFolder.Text            = RegistryManager.Instance.ReadString("OutputFolder", Environment.GetFolderPath(Environment.SpecialFolder.MyPictures));
            LangManager.Instance.Language   = RegistryManager.Instance.ReadEnum<LangManager.ELanguage>("Lang", LangManager.ELanguage.English);
            chkShowOutput.Checked           = RegistryManager.Instance.ReadBool("ShowOutputFolder");
            cmbSize.SelectedItem            = RegistryManager.Instance.ReadInt("Size", (int)cmbSize.Items[cmbSize.Items.Count-1]);

            ComboWrapper.SelectItem(cmbFormat, OutputFormat.Parse(RegistryManager.Instance.ReadString("Format")), ComboWrapper.EDefaultMode.First);
            
            refreshLang();

            listViewFiles.SmallImageList = iconManager.SmallIcons;
            listViewFiles.LargeImageList = iconManager.LargeIcons;
        }

        private void refreshLang()
        {
            LangManager li = LangManager.Instance;

            this.Text                   = li.Read(LangManager.Application, "App_Title");
            tsStatus.Text               = li.Read(LangManager.MainForm,    "Msg_Normal");
            btnAddFile.ButtonText       = li.Read(LangManager.MainForm, "AddFile");
            btnAddFolder.ButtonText     = li.Read(LangManager.MainForm, "AddFolder");
            btnAddFragment.ButtonText   = li.Read(LangManager.MainForm, "AddFragment");
            btnAddURL.ButtonText        = li.Read(LangManager.MainForm, "AddURL");
            btnSelectAll.ButtonText     = li.Read(LangManager.MainForm, "SelectAll");
            btnSelectNone.ButtonText    = li.Read(LangManager.MainForm, "SelectNone");
            btnDelFile.ButtonText       = li.Read(LangManager.MainForm, "Delete");
            btnCapture.ButtonText       = li.Read(LangManager.MainForm, "Capture");
            lblSelectoutputFormat.Text  = li.Read(LangManager.MainForm, "SelectOuputFormat");
            lblOutputFolder.Text        = li.Read(LangManager.MainForm, "OutputFolder");
            chkShowOutput.Text          = li.Read(LangManager.MainForm, "ShowFolder");
            chName.Text                 = li.Read(LangManager.MainForm, "Name");
            chPath.Text                 = li.Read(LangManager.MainForm, "Path");
            chSize.Text                 = li.Read(LangManager.MainForm, "SizeKB");
            chType.Text                 = li.Read(LangManager.MainForm, "Type");
            gbAdd.Text                  = li.Read(LangManager.MainForm, "Add");
            gbSelect.Text               = li.Read(LangManager.MainForm, "Select");
            lblSize.Text                = li.Read(LangManager.MainForm, "Size");

            fileToolStripMenuItem.Text      = li.Read(LangManager.Menu, "File");
            optionsToolStripMenuItem.Text   = li.Read(LangManager.Menu, "Options");
            helpToolStripMenuItem.Text      = li.Read(LangManager.Menu, "Help");
            languageToolStripMenuItem.Text  = li.Read(LangManager.Menu, "Language");
            exitToolStripMenuItem.Text      = li.Read(LangManager.Menu, "Exit");
            hungarianToolStripMenuItem.Text = li.Read(LangManager.Menu, "Hungarian");
            englishToolStripMenuItem.Text   = li.Read(LangManager.Menu, "English");
            germanToolStripMenuItem.Text    = li.Read(LangManager.Menu, "German");

            addFileToolStripMenuItem.Text       = li.Read(LangManager.Menu, "AddFile");
            addFolderToolStripMenuItem.Text     = li.Read(LangManager.Menu, "AddFolder");
            addFragmentToolStripMenuItem.Text   = li.Read(LangManager.Menu, "AddFragment");
            addURLToolStripMenuItem.Text        = li.Read(LangManager.Menu, "AddURL");
        }

        private void btnOutputFolderSelect_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            dlg.Description = LangManager.Instance.Read(LangManager.MainForm, "Msg_SelectOutput");
            dlg.ShowNewFolderButton = true;
            dlg.SelectedPath = txtOutputFolder.Text;
            if (dlg.ShowDialog() == DialogResult.OK)
                txtOutputFolder.Text = dlg.SelectedPath;
        }

        private void btnAddFile_Click(object sender, EventArgs e)
        {
            addFile();
        }

        private void addFile()
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = LangManager.Instance.Read(LangManager.MainForm, "Msg_HtmlFiles");
            dlg.Multiselect = true;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                addFiles2List(dlg.FileNames);
            }
        }

        private void addFolder()
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            dlg.Description = LangManager.Instance.Read(LangManager.MainForm, "Msg_SelectFolder");
            dlg.ShowNewFolderButton = false;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                addFiles2List(Directory.GetFiles(dlg.SelectedPath, "*.html"));
                addFiles2List(Directory.GetFiles(dlg.SelectedPath, "*.htm"));
            }
        }

        private void addURL()
        {
            URLForm uf = new URLForm();
            if (uf.ShowDialog() == DialogResult.OK)
            {
                HtmlFileInfo info = new HtmlFileInfo(uf.URL, FileMode.Remote);

                ListViewItem item = new ListViewItem(new string[] { 
                    info.FileName, info.FilePath, info.FileSize.ToString(), info.FileMode.ToString()
                }, -1);
                item.Tag = info;
                item.Checked = true;
                listViewFiles.Items.Add(item);
            }
        }

        private void addFragment()
        {
            FragmentForm ff = new FragmentForm();
            if (ff.ShowDialog() == DialogResult.OK)
            {
                HtmlFileInfo info = new HtmlFileInfo("", FileMode.Fragment);
                info.FileName = ff.FragmentName;
                info.HtmlContent = ff.FragmentCode;

                ListViewItem item = new ListViewItem(new string[] { 
                    info.FileName, info.FilePath, info.FileSize.ToString(), info.FileMode.ToString()
                }, -1);
                item.Tag = info;
                item.Checked = true;
                listViewFiles.Items.Add(item);
            }
        }

        private void addFiles2List(string[] files)
        {
            foreach (string file in files)
            {
                FileInfo fi = new FileInfo(file);
                if (!File.Exists(file))
                    continue;

                int idx = IconManager.GetImageIndex(file, EIconSize.Small);
                HtmlFileInfo info = new HtmlFileInfo(fi.FullName, FileMode.Local);

                using (StreamReader sr = new StreamReader(info.FilePath, Encoding.Default, true))
                    info.HtmlContent = sr.ReadToEnd();

                ListViewItem item = new ListViewItem(new string[] { 
                    info.FileName, info.FilePath, info.FileSize.ToString(), info.FileMode.ToString() }, idx);
                item.Tag = info;
                item.Checked = true;
                listViewFiles.Items.Add(item);
            }
        }

        private void listViewFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnDelFile.Enabled = (listViewFiles.SelectedItems.Count > 0);
        }

        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listViewFiles.Items.Count; i++)
                listViewFiles.Items[i].Checked = true;
        }

        private void btnSelectNone_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < listViewFiles.Items.Count; i++)
                listViewFiles.Items[i].Checked = false;
        }

        private void btnAddFolder_Click(object sender, EventArgs e)
        {
            addFolder();
        }

        private void listViewFiles_Layout(object sender, LayoutEventArgs e)
        {
            if (listViewFiles.Columns.Count > 0)
                listViewFiles.Columns[listViewFiles.Columns.Count - 2].Width = -2;
        }

        private void listViewFiles_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            ListViewHelper.SortListView(sender, e);
        }

        private void listViewFiles_ItemNumberChanged(int num)
        {
            btnCapture.Enabled = btnSelectAll.Enabled = 
            btnSelectNone.Enabled = (listViewFiles.Items.Count > 0);
        }

        private void validate()
        {
            errorProviderEx1.ClearAll();
            if (cmbFormat.SelectedItem == null)
                errorProviderEx1.SetError(cmbFormat,       LangManager.Instance.Read(LangManager.MainForm, "Msg_SelectFormat"));
            if (string.IsNullOrEmpty(txtOutputFolder.Text))
                errorProviderEx1.SetError(txtOutputFolder, LangManager.Instance.Read(LangManager.MainForm, "Msg_SelectOutput"));
            if (!Directory.Exists(txtOutputFolder.Text))
                errorProviderEx1.SetError(txtOutputFolder, LangManager.Instance.Read(LangManager.MainForm, "Msg_InvalidDirectory"));
            if (listViewFiles.Items.Count == 0)
                errorProviderEx1.SetError(listViewFiles,   LangManager.Instance.Read(LangManager.MainForm, "Msg_SelectHtmlFile"));
        }

        private void btnCapture_Click(object sender, EventArgs e)
        {
            validate();
            if (errorProviderEx1.HasErrors)
                return;

            tsStatus.Text = LangManager.Instance.Read(LangManager.MainForm, "Msg_Converting");

            List<HtmlFileInfo> files = new List<HtmlFileInfo>();
            foreach (ListViewItem item in listViewFiles.Items)
            {
                if (item.Checked)
                   files.Add(item.Tag as HtmlFileInfo);
            }

            ProcessForm pf = new ProcessForm();
            pf.Files = files;
            pf.OutputFolder = txtOutputFolder.Text;
            pf.ShowOutputFolder = chkShowOutput.Checked;
            pf.Format = (OutputFormat)cmbFormat.SelectedItem;
            pf.ResizeImage = (int)cmbSize.SelectedItem;
            pf.ShowDialog();

            //remove processed items
            int i = 0;
            while( i < listViewFiles.Items.Count)
            {
                if (listViewFiles.Items[i].Checked)
                    listViewFiles.Items.RemoveAt(i);
                else
                    i++;
            }
            tsStatus.Text = LangManager.Instance.Read(LangManager.MainForm, "Msg_Normal");
        }

        private void btnDelFile_Click(object sender, EventArgs e)
        {
            if (listViewFiles.SelectedItems != null && listViewFiles.SelectedItems.Count > 0)
            {
                foreach (ListViewItem item in listViewFiles.SelectedItems)
                    listViewFiles.Items.Remove(item);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void addFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            addFolder();
        }

        private void addFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            addFile();
        }

        private void addURLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            addURL();
        }

        private void addFragmentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            addFragment();
        }

        private void chkShowOutput_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void btnAddURL_Click(object sender, EventArgs e)
        {
            addURL();
        }

        private void btnAddFragment_Click(object sender, EventArgs e)
        {
            addFragment();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            RegistryManager.Instance.Write("OutputFolder", txtOutputFolder.Text);
            RegistryManager.Instance.Write("Lang", LangManager.Instance.Language.ToString());
            RegistryManager.Instance.Write("ShowOutputFolder", chkShowOutput.Checked);
            RegistryManager.Instance.Write("Size", cmbSize.SelectedItem);
            RegistryManager.Instance.Write("Format", cmbFormat.SelectedItem);
        }

        private void englishToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LangManager.Instance.Language = LangManager.ELanguage.English;
            refreshLang();
        }

        private void hungarianToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LangManager.Instance.Language = LangManager.ELanguage.Hungarian;
            refreshLang();
        }

        private void germanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LangManager.Instance.Language = LangManager.ELanguage.German;
            refreshLang();
        }
    }
}