using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WebsiteCapture
{
    public partial class ProcessForm : Form
    {
        public ProcessForm()
        {
            InitializeComponent();

            lblStatus.Text = ""; 
        }

        private List<HtmlFileInfo> files = new List<HtmlFileInfo>();
        public List<HtmlFileInfo> Files
        {
            set { this.files = value; }
        }

        private string outputFolder;
        public string OutputFolder
        {
            set { this.outputFolder = value; }
        }

        private OutputFormat format;
        public OutputFormat Format
        {
            set { this.format = value; }
        }

        private bool showOutputFolder;
        public bool ShowOutputFolder
        {
            set { this.showOutputFolder = value; }
        }

        private int resizeImage;
        public int ResizeImage
        {
            set { this.resizeImage = value; }
        }

        bool cancelProgress = false;

        private delegate void ProcessHtmlCallback(string html, string outputPath, FileMode mode, string msg, int resize);
        private void processHtml(string html, string outputPath, FileMode mode, string msg, int resize)
        {
            if (this.InvokeRequired)
            {
                ProcessHtmlCallback d = new ProcessHtmlCallback(processHtml);
                this.Invoke(d, new object[] { html, outputPath, mode, msg, resize });
            }
            else
            {
                switch (mode)
                {
                    case FileMode.Local:
                    case FileMode.Fragment:
                        Html2Image.ConvertHtml(html, outputPath, resize);
                        break;
                    case FileMode.Remote:
                        Html2Image.ConvertPage(html, outputPath, resize);
                        break;
                    default:
                        break;
                }
                lblStatus.Text = msg;
            }
        }

        private void bgw_DoWork(object sender, DoWorkEventArgs e)
        {
            string msg;
            for (int i = 0; i < files.Count; i++)
            {
                if (cancelProgress)
                    return;

                msg = string.Format("Processing file {0}", i + 1);

                HtmlFileInfo info = files[i];
                string imgFilePath = Path.Combine(outputFolder, info.GetImgFileName(format));

                switch (info.FileMode)
                {
                    case FileMode.Local:
                        {
                            processHtml(info.HtmlContent, imgFilePath, FileMode.Local, msg, resizeImage);
                        }
                        break;

                    case FileMode.Remote:
                        {
                            processHtml(info.FilePath, imgFilePath, FileMode.Remote, msg, resizeImage);
                        }
                        break;

                    case FileMode.Fragment:
                        {
                            processHtml(info.HtmlContent, imgFilePath, FileMode.Local, msg, resizeImage);
                        }
                        break;

                    default:
                        break;
                }

                
                bgw.ReportProgress(i + 1);
            }
        }

        private void bgw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            try
            {
                if (e.ProgressPercentage <= progressBar.MaxValue)
                    progressBar.Value = e.ProgressPercentage;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        private void bgw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.Close();
            if (!e.Cancelled && showOutputFolder)
            {
                System.Diagnostics.Process.Start(outputFolder);
            }
        }

        private void btnAbort_Click(object sender, EventArgs e)
        {
            cancelProgress = true;
            this.DialogResult = DialogResult.Cancel;
        }

        string processStatus;

        private void ProcessForm_Load(object sender, EventArgs e)
        {
            this.Text           = LangManager.Instance.Read(LangManager.ProcessForm, "Title");
            this.btnAbort.ButtonText = LangManager.Instance.Read(LangManager.Common, "Abort");
            processStatus       = LangManager.Instance.Read(LangManager.ProcessForm, "Status");

            progressBar.MaxValue = this.files.Count;
            bgw.RunWorkerAsync();
        }
    }
}