using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;

namespace WebsiteCapture
{
    public class RegistryManager
    {
        private static RegistryManager instance;
        public static RegistryManager Instance
        {
            get 
            {
                if (instance == null)
                    instance = new RegistryManager();
                return instance;
            }
        }

        const string softwareRegKey = @"Software\LCSoft\WebsiteCapture";

        private RegistryManager()
        { }

        public object ReadObject(string keyName)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    return key.GetValue(keyName);
                }
            }
            return null;
        }

        public object ReadObject(string keyName, object defaultValue)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    return key.GetValue(keyName);
                }
            }
            return defaultValue;
        }

        public string ReadString(string keyName)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                        return obj.ToString();
                }
            }
            return null;
        }

        public string ReadString(string keyName, string defaultValue)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                        return obj.ToString();
                }
            }
            return defaultValue;
        }

        public bool ReadBool(string keyName)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                    {
                        try
                        {
                            return bool.Parse(obj.ToString());
                        }
                        catch
                        { }
                    }
                }
            }
            return false;
        }

        public int ReadInt(string keyName, int defaultValue)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                    {
                        try
                        {
                            return int.Parse(obj.ToString());
                        }
                        catch
                        { }
                    }
                }
            }
            return defaultValue;
        }

        public float ReadFloat(string keyName, float defaultValue)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                    {
                        try
                        {
                            return float.Parse(obj.ToString());
                        }
                        catch
                        { }
                    }
                }
            }
            return defaultValue;
        }

        public T ReadEnum<T>(string keyName, T defaultValue)
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey))
            {
                if (key != null)
                {
                    object obj = key.GetValue(keyName);
                    if (obj != null)
                    {
                        try
                        {
                            return (T)Enum.Parse(typeof(T), obj.ToString());
                        }
                        catch
                        { }
                    }
                }
            }
            return defaultValue;
        }

        public void Write(string keyName, object value)
        {
            if (value != null)
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(softwareRegKey, true);
                if (key == null)
                    key = Registry.LocalMachine.CreateSubKey(softwareRegKey);
                key.SetValue(keyName, value.ToString());
                key.Close();
            }
        }
    }
}
